Package: HDTD
Type: Package
Title: Statistical Inference about the Mean Matrix and the Covariance
        Matrices in High-Dimensional Transposable Data (HDTD)
Version: 1.44.0
Authors@R: c(
    person(given = "Anestis",
           family = "Touloumis",
           email = "A.Touloumis@brighton.ac.uk",
           role = c("cre","aut"),
           comment = c(ORCID = "0000-0002-5965-1639")),
    person(given = "John C.",
           family = "Marioni",
           role = "aut",
           comment = c(ORCID = "0000-0001-9092-0852")),
    person(given = "Simon",
           family = "Tavar\\'{e}",
           role = "aut",
           comment = c(ORCID = "0000-0002-3716-4952"))
           )
Depends: R (>= 4.1)
Suggests: knitr, rmarkdown
Imports: stats, Rcpp (>= 1.0.1)
LinkingTo: Rcpp, RcppArmadillo
Description: Characterization of intra-individual variability using physiologically relevant measurements provides important insights into fundamental biological questions ranging from cell type identity to tumor development. For each individual, the data measurements can be written as a matrix with the different subsamples of the individual recorded in the columns and the different phenotypic units recorded in the rows. Datasets of this type are called high-dimensional transposable data. The HDTD package provides functions for conducting statistical inference for the mean relationship between the row and column variables and for the covariance structure within and between the row and column variables.
License: GPL-3
biocViews: DifferentialExpression, Genetics, GeneExpression,
        Microarray, Sequencing, StatisticalMethod, Software
VignetteBuilder: knitr
URL: http://github.com/AnestisTouloumis/HDTD
BugReports: http://github.com/AnestisTouloumis/HDTD/issues
LazyData: false
NeedsCompilation: yes
RoxygenNote: 7.1.1
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/HDTD
git_branch: RELEASE_3_22
git_last_commit: b83dbae
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
Packaged: 2025-10-30 04:24:43 UTC; biocbuild
Author: Anestis Touloumis [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-5965-1639>),
  John C. Marioni [aut] (ORCID: <https://orcid.org/0000-0001-9092-0852>),
  Simon Tavar\'{e} [aut] (ORCID: <https://orcid.org/0000-0002-3716-4952>)
Maintainer: Anestis Touloumis <A.Touloumis@brighton.ac.uk>

# Generated by roxygen2: do not edit by hand

S3method(print,covmat.hat)
S3method(print,covmat.ts)
S3method(print,meanmat.hat)
S3method(print,meanmat.ts)
export(centerdata)
export(covmat.hat)
export(covmat.ts)
export(meanmat.hat)
export(meanmat.ts)
export(orderdata)
export(transposedata)
import(Rcpp)
importFrom(stats,na.omit)
importFrom(stats,pnorm)
useDynLib(HDTD, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# HDTD: Analyzing High-Dimensional Transposable Data

[![Travis-CI Build
Status](https://travis-ci.org/AnestisTouloumis/HDTD.svg?branch=master)](https://travis-ci.org/AnestisTouloumis/HDTD)
[![Project Status: Active The project has reached a stable, usable state
and is being actively
developed.](http://www.repostatus.org/badges/latest/active.svg)](http://www.repostatus.org/#active)

## Installation

You can install the release version of `HDTD`:

``` r
if (!requireNamespace("BiocManager", quietly = TRUE)) install.packages("BiocManager")
BiocManager::install("HDTD")
```

The source code for the release version of `HDTD` is available on
Bioconductor at:

-   <http://bioconductor.org/packages/release/bioc/html/HDTD.html>

Or you can install the development version of `HDTD`:

``` r
if (!requireNamespace("devtools", quietly = TRUE)) install.packages("devtools")
devtools::install_github("AnestisTouloumis/HDTD")
```

To use `HDTD`, you should load the package as follows:

``` r
library("HDTD")
```

## Usage

This package offers functions to estimate and test the matrix parameters
of transposable data in high-dimensional settings. The term
*transposable data* refers to datasets that are structured in a matrix
form such that both the rows and columns correspond to variables of
interest and dependencies are expected to occur *among* rows, *among*
columns and *between* rows and columns. For example, consider microarray
studies in genetics where multiple RNA samples across different tissues
are available per subject. In this case, a data matrix can be created
with row variables the genes, column variables the tissues and
measurements the corresponding expression levels. We expect dependencies
to occur among genes, among tissues and between genes and tissues. For
more examples of transposable data see references in Touloumis, Marioni
and Tavaré (2021), Touloumis, Tavaré and Marioni (2015) and Touloumis,
Marioni and Tavaré (2016).

There are four core functions:

-   `meanmat.hat` to estimate the mean matrix of the transposable data,
-   `meanmat.ts` to test the overall mean of the row (column) variables
    across groups of column (row) variables,
-   `covmat.hat` to estimate the row and column covariance matrix,
-   `covmat.ts` to test the sphericity, identity and diagonality
    hypothesis test for the row/column covariance matrix.

There are also three utility functions:

-   `transposedata` for interchanging the role of rows and columns,
-   `centerdata` for centering the transposable data around their mean
    matrix,
-   `orderdata` for rearranging the order of the row and/or column
    variables.

## Example

We replicate the analysis that can be found in the vignette based on the
mouse dataset

``` r
data(VEGFmouse)
```

This dataset contains expression levels for 40 mice. For each mouse, the
expression levels of 46 genes (rows) that belong to the vascular
endothelial growth factor signalling pathway were measured across 9
tissues (adrenal gland, cerebrum, hippocampus, kidney, lung, muscle,
spinal cord, spleen and thymus) that are displayed in the columns.

One can estimate the mean relationship of the gene expression levels
across the 9 tissues

``` r
sample_mean <- meanmat.hat(datamat = VEGFmouse, N = 40)
sample_mean
#> ESTIMATION OF THE MEAN MATRIX 
#> Sample size      = 40 
#> Row variables    = 46 
#> Column variables = 9 
#> 
#> Estimated mean matrix [1:5, 1:5] =
#>       adrenal.1 cerebrum.1 hippocampus.1 kidney.1  lung.1
#> Akt1     0.8399     1.2157        1.0597   1.1469  1.2673
#> Akt2    -0.2333    -0.6201       -0.3881  -0.5524 -0.5359
#> Akt3    -1.0856    -0.4351       -0.5490  -0.2534 -0.6091
#> Arnt     0.1089     0.1898        0.0968   0.2551 -0.1171
#> Casp9    0.0877     0.2600        0.4812   0.3203  0.7416
```

and test whether the overall gene expression is constant across the 9
tissues:

``` r
tissue_mean_test <- meanmat.ts(datamat = VEGFmouse, N = 40, group.sizes = 9)
tissue_mean_test
#> MEAN MATRIX TEST 
#> Sample size      = 40 
#> Row variables    = 46 
#> Column variables = 9 
#> 
#> H_0: a constant mean vector across columns 
#> H_1: not H_0 
#> 
#> Test statistic = 373.5277, p-value < 0.0001
```

In this case, the overall gene expression is not conserved.

To analyze the gene-wise and tissue-wise dependence structure, one needs
to estimate the two covariance matrices:

``` r
est_cov_mat <- covmat.hat(datamat = VEGFmouse, N = 40)
est_cov_mat
#> ESTIMATION OF THE ROW AND/OR COLUMN COVARIANCE MATRIX 
#> Sample size      = 40 
#> Row variables    = 46 
#> Column variables = 9 
#> Shrinking        = Both sets of variables 
#> Centered data    = FALSE 
#> 
#> ROW VARIABLES
#> Estimated optimal intensity = 0.0115 
#> Estimated covariance matrix [1:5, 1:5] =
#>          Akt1    Akt2    Akt3    Arnt   Casp9
#> Akt1   0.4139 -0.0248  0.0420 -0.0010  0.1084
#> Akt2  -0.0248  0.3341 -0.0240 -0.0029 -0.0151
#> Akt3   0.0420 -0.0240  0.6954  0.1733 -0.0168
#> Arnt  -0.0010 -0.0029  0.1733  0.4746  0.0850
#> Casp9  0.1084 -0.0151 -0.0168  0.0850  0.5337
#> 
#> COLUMN VARIABLES
#> Estimated optimal intensity = 0.3341 
#> Estimated covariance matrix [1:5, 1:5] =
#>               adrenal.1 cerebrum.1 hippocampus.1 kidney.1  lung.1
#> adrenal.1        0.0368    -0.0006        0.0001  -0.0006  0.0010
#> cerebrum.1      -0.0006     0.0432       -0.0002   0.0000 -0.0034
#> hippocampus.1    0.0001    -0.0002        0.0266   0.0019  0.0000
#> kidney.1        -0.0006     0.0000        0.0019   0.0317  0.0012
#> lung.1           0.0010    -0.0034        0.0000   0.0012  0.0809
```

Finally, the package allows users to perform hypothesis tests for the
covariance matrix of the genes

``` r
genes_cov_test <- covmat.ts(VEGFmouse, N = 40)
genes_cov_test
#> HYPOTHESES TESTS FOR THE ROW COVARIANCE MATRIX 
#> Sample size      = 40 
#> Row variables    = 46 
#> Column variables = 9 
#> Centered data    = FALSE 
#> 
#> Diagonality hypothesis test:
#> Test Statistic = 8.6324, p-value < 0.0001
#> 
#> Sphericity hypothesis test:
#> Test Statistic = 132.8086, p-value < 0.0001
#> 
#> Identity hypothesis test:
#> Test Statistic = 30.3864, p-value < 0.0001
```

and of the tissues:

``` r
tissues_cov_test <- covmat.ts(VEGFmouse, N = 40, voi = "columns")
tissues_cov_test
#> HYPOTHESES TESTS FOR THE COLUMN COVARIANCE MATRIX 
#> Sample size      = 40 
#> Row variables    = 46 
#> Column variables = 9 
#> Centered data    = FALSE 
#> 
#> Diagonality hypothesis test:
#> Test Statistic = 1.4866, p-value = 0.0686
#> 
#> Sphericity hypothesis test:
#> Test Statistic = 10.2122, p-value < 0.0001
#> 
#> Identity hypothesis test:
#> Test Statistic = 38.0811, p-value < 0.0001
```

At a 5% significance level, it appears that the genes are correlated but
we do not have enough evidence to reject the hypothesis that the tissues
are uncorrelated.

## Getting help

The statistical methods implemented in `HDTD` are described in
Touloumis, Marioni and Tavaré (2021), Touloumis, Tavaré and Marioni
(2015) and Touloumis, Marioni and Tavaré (2016). Detailed examples of
`HDTD` can be found in Touloumis, Marioni and Tavaré (2016) or in the
vignette:

``` r
browseVignettes("HDTD")
```

## How to cite


    Please use the following guidelines for citing `HDTD' in publication:

    To cite the mean matrix hypothesis testing methodology, please use

      Touloumis, A., Tavar\'{e}, S. and Marioni, J.C. (2015). Testing the
      Mean Matrix in High-Dimensional Transposable Data. Biometrics 71 (1),
      157-166

    A BibTeX entry for LaTeX users is

      @Article{,
        title = {Testing the Mean Matrix in High-Dimensional Transposable Data},
        author = {Anestis Touloumis and Simon Tavar\'{e} and John C. Marioni},
        journal = {Biometrics},
        year = {2015},
        volume = {71},
        issue = {1},
        pages = {157--166},
        url = {http://onlinelibrary.wiley.com/doi/10.1111/biom.12257/full},
      }

    To cite the covariance matrix hypothesis testing methodology, please
    use

      Touloumis, A., Marioni, J.C. and Tavar\'{e}, S. (2019+). Hypothesis
      Testing for the Covariance Matrix in High-Dimensional Transposable
      Data with Kronecker Product Dependence Structure. Statistica Sinica

    A BibTeX entry for LaTeX users is

      @Article{,
        title = {Hypothesis Testing for the Covariance Matrix in 
             High-Dimensional Transposable Data with Kronecker Product Dependence Structure},
        author = {Anestis Touloumis and John C. Marioni and Simon Tavar\'{e}},
        journal = {Statistica Sinica},
        year = {2019+},
        url = {http://www3.stat.sinica.edu.tw/ss_newpaper/SS-2018-0268_na.pdf},
      }

    To cite HDTD or the estimation method for the covariance matrices,
    please use

      Touloumis, A., Marioni, J.C. and Tavar\'{e}, S. (2016). HDTD:
      Analyzing multi-tissue gene expression data. Bioinformatics 32 (14),
      2193-2195

    A BibTeX entry for LaTeX users is

      @Article{,
        title = {HDTD: Analyzing multi-tissue gene expression data},
        author = {Anestis Touloumis and John C. Marioni and Simon Tavar\'{e}},
        journal = {Bioinformatics},
        year = {2016},
        volume = {32},
        issue = {14},
        pages = {2193--2195},
        url = {https://doi.org/10.1093/bioinformatics/btw224},
      }

# References

<div id="refs" class="references csl-bib-body hanging-indent">

<div id="ref-Touloumis2016" class="csl-entry">

Touloumis, A., Marioni, J.C. and Tavaré, S. (2016) HDTD: Analyzing
Multi-tissue Gene Expression Data. *Bioinformatics*, **32**, 2193–2195.

</div>

<div id="ref-Touloumis2013" class="csl-entry">

Touloumis, A., Marioni, J.C. and Tavaré, S. (2021) Hypothesis Testing
for the Covariance Matrix in High-Dimensional Transposable Data with
Kronecker Product Dependence Structure. *Statistica Sinica*, **31**,
1309–1329.

</div>

<div id="ref-Touloumis2015" class="csl-entry">

Touloumis, A., Tavaré, S. and Marioni, J.C. (2015) Testing the Mean
Matrix in High-Dimensional Transposable Data. *Biometrics*, **71**,
157–166.

</div>

</div>

#' Nonparametric Tests for the Mean Matrix
#'
#' This function performs hypothesis testing for the mean matrix.
#'
#' It is assumed that there are \code{nrow(datamat)} row variables and
#' \code{ncol(datamat)}/\code{N} column variables in \code{datamat}. Further,
#' \code{datamat} should be written in such a way that every
#' \code{ncol(datamat)}/\code{N} consecutive columns belong to the same subject
#' and the order of the column variables in each block is preserved across
#' subjects.
#'
#' @aliases meanmat.ts print.meanmat.ts
#' @param datamat numeric matrix containing the transposable data.
#' @param N positive integer number indicating the sample size, i.e., the
#' number of subjects.
#' @param group.sizes numeric vector indicating the group sizes under the null
#' hypothesis.
#' @param voi character indicating if the test will be applied to the row or
#' column variables. Options include '\code{rows}' or '\code{columns}'.
#' @return Returns a list with components: \item{statistic}{the value of the
#' test statistic.} \item{p.value}{the corresponding p-value.} \item{voi}{the
#' set of variables that the test was applied to.} \item{n.groups}{the number
#' of groups under the null hypothesis.} \item{group.sizes}{the size of each
#' group under the null hypothesis.} \item{N}{the sample size.}
#' \item{n.rows}{the number of row variables.} \item{n.cols}{the number of
#' column variables.}
#' @author Anestis Touloumis
#' @references Touloumis, A., Tavare, S. and Marioni, J. C. (2015) Testing the
#' Mean Matrix in High-Dimensional Transposable Data.
#' \emph{Biometrics} \bold{71}, 157--166.
#'
#' @examples
#' data(VEGFmouse)
#' ## Testing conservation of the overall gene expression across tissues.
#' tissues_mean_test <- meanmat.ts(datamat = VEGFmouse, N = 40, group.sizes = 9)
#' tissues_mean_test
#' ## Testing if the adrenal and the cerebrum tissues have the same mean vector.
#' test2 <- meanmat.ts(VEGFmouse, N = 40, group.sizes = c(2, rep(1,7)))
#' test2
#' @export
meanmat.ts <- function(datamat, N, group.sizes, voi = "columns") {
    if (!is.matrix(datamat))
        datamat <- as.matrix(datamat)
    datamat <- na.omit(datamat)
    N <- as.numeric(N)
    if (length(N) != 1 | ((N - round(N)) != 0) | (N <= 3))
        stop("'N' must be a positive integer number greater than 3")
    group.sizes <- as.numeric(group.sizes)
    if (any(group.sizes <= 0) || any((group.sizes - round(group.sizes)) != 0))
        stop("'group.sizes' must be a vector of positive integer numbers")
    voi <- as.character(voi)
    if (voi != "columns" & voi != "rows")
        stop("'voi' must be either 'rows' or 'columns'")
    p1 <- nrow(datamat)
    p2 <- ncol(datamat) / N
    if ((p2 - round(p2)) != 0)
        stop("The number of column variables is not a positive integer number")
    if (voi == "columns" & sum(group.sizes) != p2)
        stop("Total group sizes is less than the number of column variables")
    if (voi == "rows" & sum(group.sizes) != p1)
        stop("Total group sizes is less than the number of row variables")
    if (all(group.sizes == 1))
        stop("The size of at least one group must be greater than one")
    if (any(group.sizes == 1)) {
        rmvars <- cumsum(group.sizes)[group.sizes == 1]
        order.rows <- order.cols <- NULL
        if (voi == "rows")
            order.rows <- seq_len(p1)[-rmvars] else order.cols <- seq_len(p2)[-rmvars]
        datamat <- orderdata(datamat, N, order.rows, order.cols)
        group.sizes1 <- group.sizes[group.sizes != 1]
        projmat <- projectionmatrix(cumsum(group.sizes1))
    } else projmat <- projectionmatrix(cumsum(group.sizes))
    ts <- meanmat.ts.generic(datamat, N, projmat, voi)
    ans <- list(statistic = ts, p.value = 1 - pnorm(ts), voi = voi,
                n.groups = length(group.sizes), group.sizes = group.sizes,
                N = N, n.rows = p1, n.cols = p2)
    class(ans) <- "meanmat.ts"
    ans
}

#' Interchanging the Row and Column Variables in Transposable Data
#'
#' This function interchanges the row and column variables in transposable data
#' so that the original row variables will be treated as column variables and
#' the original column variables as row variables.
#'
#' It is assumed that there are \code{nrow(datamat)} row variables and
#' \code{ncol(datamat)}/\code{N} column variables in \code{datamat}. Further,
#' \code{datamat} should be written in such a way that every
#' \code{ncol(datamat)}/\code{N} consecutive columns belong to the same subject
#' and the order of the column variables in each block is preserved across
#' subjects.
#'
#' @param datamat numeric matrix containing the transposable data.
#' @param N positive integer number indicating the sample size, i.e., the
#' number of subjects.
#' @return Returns a matrix with \code{ncol(datamat)} rows and
#' \code{nrow(datamat)}/\code{N} columns.
#' @author Anestis Touloumis
#' @seealso \code{\link{centerdata}} and \code{\link{orderdata}}.
#' @examples
#' data(VEGFmouse)
#' ## Transposing the VEGF dataset.
#' VEGFtr <- transposedata(datamat = VEGFmouse, N = 40)
#' @export transposedata
transposedata <- function(datamat, N) {
    if (!is.matrix(datamat))
        datamat <- as.matrix(datamat,
                            dimnames =
                                list(rownames(datamat), colnames(datamat)))
    datamat <- na.omit(datamat)
    row.vars.names <- rownames(datamat)
    col.vars.names <- colnames(datamat)
    N <- as.numeric(N)
    if (length(N) != 1 | ((N - round(N)) != 0) | (N <= 0))
        stop("'N' must be a positive integer number")
    p2 <- ncol(datamat) / N
    if ((p2 - round(p2)) != 0)
        stop("The number of column variables is not a positive integer number")
    ans <- transposedatamatrix(datamat, N)
    rownames(ans) <- col.vars.names[seq_len(p2)]
    colnames(ans) <- paste(row.vars.names,
                            rep(seq_len(N), each = ncol(ans) / N), sep = ".")
    ans
}

#' @export
print.covmat.ts <- function(x, ...) {
    print_vars <- ifelse(x$variables == "Rows", "ROW", "COLUMN")
    cat("HYPOTHESES TESTS FOR THE", print_vars, "COVARIANCE MATRIX", "\n")
    cat("Sample size      =", x$N, "\n")
    cat("Row variables    =", x$n.rows, "\n")
    cat("Column variables =", x$n.cols, "\n")
    cat("Centered data    =", x$centered, "\n")
    cat("\nDiagonality hypothesis test:\n")
    if (x$diagonality.ts$p.value < 1e-04)
        cat("Test Statistic = ", round(x$diagonality.ts$statistic, 4),
            ", p-value < 0.0001\n", sep = "")
    if (x$diagonality.ts$p.value > 1e-04)
        cat("Test Statistic = ", round(x$diagonality.ts$statistic, 4),
            ", p-value = ", round(x$diagonality.ts$p.value, 4), "\n", sep = "")
    cat("\nSphericity hypothesis test:\n")
    if (x$sphericity.ts$p.value < 1e-04)
        cat("Test Statistic = ", round(x$sphericity.ts$statistic, 4),
            ", p-value < 0.0001\n", sep = "")
    if (x$sphericity.ts$p.value > 1e-04)
        cat("Test Statistic = ", round(x$sphericity.ts$statistic, 4),
            ", p-value = ", round(x$sphericity.ts$p.value, 4), "\n", sep = "")
    cat("\nIdentity hypothesis test:\n")
    if (x$identity.ts$p.value < 1e-04)
        cat("Test Statistic = ", round(x$identity.ts$statistic, 4),
            ", p-value < 0.0001\n", sep = "")
    if (x$identity.ts$p.value > 1e-04)
        cat("Test Statistic = ", round(x$identity.ts$statistic, 4),
            ", p-value = ",
            round(x$identity.ts$p.value, 4), "\n", sep = " ")
}

#' @export
print.meanmat.hat <- function(x, ...) {
    cat("ESTIMATION OF THE MEAN MATRIX", "\n")
    cat("Sample size      =", x$N, "\n")
    cat("Row variables    =", x$n.rows, "\n")
    cat("Column variables =", x$n.cols, "\n")
    cat("\nEstimated mean matrix [1:5, 1:5] =\n")
    print(round(x$estmeanmat[seq_len(min(5, x$n.rows)), 
                             seq_len(min(5, x$n.cols))], 
                4))
}

#' Nonparametric Tests for the Row or Column Covariance Matrix
#'
#' Testing the sphericity, identity and diagonality hypotheses for
#' the row or column covariance matrix.
#'
#' It is assumed that there are \code{nrow(datamat)} row variables and
#' \code{ncol(datamat)}/\code{N} column variables in \code{datamat}. Further,
#' \code{datamat} should be written in such a way that every
#' \code{ncol(datamat)}/\code{N} consecutive columns belong to the same subject
#' and the order of the column variables in each block is preserved across
#' subjects.
#'
#' The tests are nonparametric and thus robust to some departures from the
#' matrix-variate normal model.
#'
#' @aliases covmat.ts print.covmat.ts
#' @param datamat numeric matrix containing the transposable data.
#' @param N positive integer number indicating the sample size, i.e., the
#' number of subjects.
#' @param voi character indicating if the test should be applied on the row or
#' column covariance matrix. Options include '\code{rows}' or '\code{columns}'.
#' @param centered logical indicating if the transposable data are centered.
#' Options include \code{TRUE} or \code{FALSE}.
#' @return It returns a list with components:
#' \item{diagonality.ts}{a list
#' containing the test statistic and p-value of the
#' diagonality hypothesis test.}
#'\item{sphericity.ts}{a list
#' containing the test statistic and p-value of the
#' sphericity hypothesis test.}
#' \item{identity.ts}{a list containing the test statistic and p-value of the
#' identity hypothesis test.}
#' \item{N}{the sample size.}
#'\item{n.rows}{the number of row
#' variables.}
#' \item{n.cols}{the number of column variables.}
#' \item{variables}{character indicating if the tests were applied to the row
#' or column covariance matrix.}
#' \item{centered}{logical indicating if the
#' transposable data were centered.}
#' @author Anestis Touloumis
#' @references Touloumis, A., Marioni, J.C. and Tavare, S. (2021) Hypothesis
#' Testing for the Covariance Matrix in High-Dimensional Transposable Data with
#' Kronecker Product Dependence Structure. \emph{Statistica Sinica} \bold{31},
#' 1309--1329.
#' @examples
#' data(VEGFmouse)
#' ## Hypothesis tests for the covariance matrix of the genes (rows).
#' genes_cov_test <- covmat.ts(datamat = VEGFmouse, N = 40)
#' genes_cov_test
#' ## Hypothesis tests for the covariance matrix of the tissues (columns).
#' tissues_cov_test <- covmat.ts(datamat = VEGFmouse, N = 40, voi = 'columns')
#' tissues_cov_test
#' @export covmat.ts
covmat.ts <- function(datamat = datamat, N = N, voi = "rows",
                        centered = FALSE) {
    if (!is.matrix(datamat))
        datamat <- as.matrix(datamat)
    datamat <- na.omit(datamat)
    N <- as.numeric(N)
    if (length(N) != 1 | ((N - round(N)) != 0) | (N <= 0))
        stop("'N' must be a positive integer number")
    voi <- as.character(voi)
    if (voi != "columns" & voi != "rows")
        stop("'voi' must be either 'rows' or 'columns'")
    centered <- as.logical(centered)
    if (centered != TRUE & centered != FALSE)
        stop("'centered' must be either 'TRUE' or 'FALSE'")
    if (!centered & N <= 3)
        stop("'N' must be greater than or equal to 4")
    p1 <- nrow(datamat)
    p2 <- ncol(datamat) / N
    if ((p2 - round(p2)) != 0)
        stop("The number of column variables is not a positive integer number")
    if (voi == "columns")
        datamat <- transposedata(datamat, N)
    ts <- covmat.ts.generic(datamat, N, centered)
    ans <- list(diagonality.ts = list(statistic = ts$Dtest,
                                        p.value = 1 - pnorm(ts$Dtest)),
                sphericity.ts = list(statistic = ts$Utest,
                                     p.value = 1 - pnorm(ts$Utest)),
                identity.ts = list(statistic = ts$Vtest,
                                   p.value = 1 - pnorm(ts$Vtest)),
                N = N, n.rows = p1, n.cols = p2,
                variables = if (voi == "rows") "Rows" else "Columns",
                centered = centered)
    class(ans) <- "covmat.ts"
    ans
}

#' Estimation and Hypothesis Testing in High-Dimensional Transposable Data
#'
#' The package HDTD offers functions to estimate and test the matrix parameters
#' of transposable data in high-dimensional settings.
#'
#' The term transposable data refers to datasets that are structured in a
#' matrix form such that both the rows and columns correspond to variables of
#' interest. For example, consider microarray studies in genetics where
#' multiple RNA samples across different tissues are available per subject. In
#' this case, a data matrix can be created with row variables the genes, column
#' variables the tissues and measurements the corresponding expression levels.
#'
#' The function \code{\link{meanmat.hat}} estimates the mean matrix of the
#' transposable data.
#'
#' The mean relationship of the row and column variables can be tested using
#' the function \code{\link{meanmat.ts}}. The implemented test is nonparametric
#' and not seriously restricted by the dependence structure among and/or
#' between the row and column variables.
#' See \cite{Touloumis et al. (2015)} for more details.
#'
#' The function \code{\link{covmat.hat}} provides Stein-type shrinkage
#' estimators for the row covariance matrix and/or for the column covariance
#' matrix under a matrix-variate normal model.
#' See \cite{Touloumis et al. (2016)} for more details.
#'
#' The sphericity and identity hypothesis for the row or column covariance
#' matrix can be tested using the function \code{\link{covmat.ts}}. Both tests
#' are nonparametric, i.e., they do not rely on a normality assumption.
#' See \cite{Touloumis et al. (2017)} for more details.
#'
#' There are three utility functions that allow the user to change to
#' interchange the role of row and column variables
#' (\code{\link{transposedata}}), to center the transposable data
#' (\code{\link{centerdata}}) or to rearrange the order of the row and/or
#' column variables (\code{\link{orderdata}}).
#'
#' @name HDTD-package
#' @aliases HDTD-package HDTD
#' @docType package
#' @author Anestis Touloumis, John Marioni, Simon Tavare.
#'
#' Maintainer: Anestis.Touloumis <A.Touloumis@@brighton.ac.uk>
#' @references Touloumis, A., Tavare, S. and Marioni, J. C. (2015) Testing the
#' Mean Matrix in High-Dimensional Transposable Data.
#' \emph{Biometrics} \bold{71}, 157--166.
#'
#' Touloumis, A., Marioni, J. C. and Tavare, S. (2016) HDTD: Analyzing
#' multi-tissue gene expression data. \emph{Bioinformatics}
#' \bold{32}, 2193--2195.
#'
#' Touloumis, A., Marioni, J. C. and Tavare, S. (2021) Hypothesis Testing for
#' the Covariance Matrix in High-Dimensional Transposable Data with Kronecker
#' Product Dependence Structure. \emph{Statistica Sinica} \bold{31}, 1309--1329.
#' @keywords package
#' @examples
#' data(VEGFmouse)
#' ## The sample mean matrix.
#' sample_mean <- meanmat.hat(datamat = VEGFmouse, N = 40)
#' sample_mean
#' ## Testing conservation of the overall gene expression across tissues.
#' tissues_mean_test <- meanmat.ts(datamat = VEGFmouse, N = 40, group.sizes = 9)
#' tissues_mean_test
#' # Estimating the gene and column covariance matrices.
#' est_cov_mat <- covmat.hat(datamat = VEGFmouse, N = 40)
#' est_cov_mat
#' ## Hypothesis tests for the covariance matrix of the genes (rows).
#' genes_cov_test <- covmat.ts(datamat = VEGFmouse, N = 40)
#' genes_cov_test
#' ## Hypothesis tests for the covariance matrix of the tissues (columns).
#' tissues_cov_test <- covmat.ts(datamat = VEGFmouse, N = 40, voi = 'columns')
#' tissues_cov_test
#' @useDynLib HDTD, .registration = TRUE
#' @import Rcpp
#' @importFrom stats na.omit pnorm
NULL

#' Vascular Endothelial Growth Factor Mouse Dataset
#'
#' Log2 normalized mouse gene expression data in the vascular endothelial
#' growth factor signalling pathway across multiple tissues.
#'
#'
#' @name VEGFmouse
#' @docType data
#' @format A data frame with 46 rows and 360 columns. The rows corresponds to
#' 46 genes in the VEGF signalling pathway. The column names indicate the mouse
#' and the tissue on which gene expression levels were measured.  Since there
#' are 40 mice and 9 tissues, we have a total of 360 columns. Every 9
#' consecutive columns belong to the same mouse and the tissues are ordered in
#' the same way in each mouse.
#' @source Zahn et al. (2007). AGEMAP: A gene expression database for aging in
#' mice. \emph{PLoS Genetics} \bold{3}, e201.
#' @keywords datasets
#' @examples
#' data(VEGFmouse)
#' ## Check the order of the tissues from the first mouse.
#' colnames(VEGFmouse[,1:9])
NULL

#' @export
print.covmat.hat <- function(x, ...) {
    cat("ESTIMATION OF THE ROW AND/OR COLUMN COVARIANCE MATRIX", "\n")
    cat("Sample size      =", x$N, "\n")
    cat("Row variables    =", x$n.rows, "\n")
    cat("Column variables =", x$n.cols, "\n")
    cat("Shrinking        =", x$shrink, "\n")
    cat("Centered data    =", x$centered, "\n")
    if (!is.null(x$rows.covmat)) {
        cat("\nROW VARIABLES\n")
        if (!is.null(x$rows.intensity))
            cat("Estimated optimal intensity =",
                round(x$rows.intensity, 4), "\n")
        cat("Estimated covariance matrix [1:5, 1:5] =\n")
        print(round(x$rows.covmat[seq_len(min(5, x$n.rows)), 
                                  seq_len(min(5, x$n.rows))], 
                    4))
    }
    if (!is.null(x$cols.covmat)) {
        cat("\nCOLUMN VARIABLES\n")
        if (!is.null(x$cols.intensity))
            cat("Estimated optimal intensity =",
                round(x$cols.intensity, 4), "\n")
        cat("Estimated covariance matrix [1:5, 1:5] =\n")
        print(round(x$cols.covmat[seq_len(min(5, x$n.cols)),
                                  seq_len(min(5, x$n.cols))], 4))
    }
}

#' Reordering Row and Column Variables
#'
#' This utility function rearranges the row and/or the column variables in a
#' desired order.
#'
#' It is assumed that there are \code{nrow(datamat)} row variables and
#' \code{ncol(datamat)}/\code{N} column variables in \code{datamat}. Further,
#' \code{datamat} should be written in such a way that every
#' \code{ncol(datamat)}/\code{N} consecutive columns belong to the same subject
#' and the order of the column variables in each block is preserved across
#' subjects.
#'
#' @param datamat numeric matrix containing the transposable data.
#' @param N positive integer number indicating the sample size, i.e., the
#' number of subjects.
#' @param order.rows numeric vector displaying the desired order of the row
#' variables.
#' @param order.cols numeric vector displaying the desired order of the column
#' variables.
#' @return Returns a matrix of the same size as \code{datamat}.
#' @author Anestis Touloumis
#' @seealso \code{\link{meanmat.ts}} and \code{\link{meanmat.hat}}.
#' @examples
#' data(VEGFmouse)
#' set.seed(1)
#' tissuesold <-  colnames(VEGFmouse[ ,1:9])
#' ## Suppose that you want to order the tissues in the folowing order.
#' tissuesnew <- colnames(VEGFmouse[ ,1:9])[sample(9)]
#' tissuesnew
#' ## To do this, create a numeric vector with the desired order.
#' ordtis <- pmatch(tissuesnew, tissuesold)
#' VEGFmousenew <- orderdata(datamat = VEGFmouse, N = 40, order.cols = ordtis)
#' colnames(VEGFmousenew)[1:9]
#' @export orderdata
orderdata <- function(datamat, N, order.rows = NULL, order.cols = NULL) {
    if (!is.matrix(datamat))
        datamat <- as.matrix(datamat,
                             dimnames =
                                 list(rownames(datamat), colnames(datamat)))
    datamat <- na.omit(datamat)
    row.vars.names <- rownames(datamat)
    col.vars.names <- colnames(datamat)
    N <- as.numeric(N)
    if (length(N) != 1 | ((N - round(N)) != 0) | (N <= 0))
        stop("'N' must be a positive integer number")
    p2 <- ncol(datamat) / N
    if ((p2 - round(p2)) != 0)
        stop("The number of column variables is not a positive integer number")
    if ((is.null(order.rows) & is.null(order.cols)))
        stop("At least one of 'order.rows' or 'order.cols' must be provided")
    if (!is.null(order.rows)) {
        if (!is.numeric(order.rows))
            order.rows <- as.numeric(order.rows)
        if (length(unique(order.rows)) < length(order.rows))
            stop("'order.rows' contains duplicated row variables")
        if (any((order.rows - round(order.rows)) != 0) || any(order.rows <= 0))
            stop("'order.rows' must be a vector of positive integer numbers")
        ans <- datamat[order.rows, ]
        if (!is.null(row.vars.names))
            rownames(ans) <- row.vars.names[order.rows]
    }
    if (!is.null(order.cols)) {
        if (!is.numeric(order.cols))
            order.cols <- as.numeric(order.cols)
        if (length(unique(order.cols)) < length(order.cols))
            stop("'order.cols' contains duplicated column variables")
        if (any((order.cols - round(order.cols)) != 0) || any(order.cols <= 0))
            stop("'order.cols' must be a vector of positive integer numbers")
        if (!is.null(order.rows)) {
            ans <- transposedata(ans, N)[order.cols, ]
            } else {
                ans <- transposedata(datamat, N)[order.cols, ]
                }
        ans <- transposedata(ans, N)
        if (!is.null(col.vars.names))
            colnames(ans) <- paste(col.vars.names[order.cols],
                                    rep(seq_len(N), each = ncol(ans) / N),
                sep = ".")
    }
    ans
}

#' Estimation of the Row and of the Column Covariance Matrices.
#'
#' This function provides the row and/or column covariance matrix estimators.
#'
#' It is assumed that there are \code{nrow(datamat)} row variables and
#' \code{ncol(datamat)}/\code{N} column variables in \code{datamat}. Further,
#' \code{datamat} should be written in such a way that every
#' \code{ncol(datamat)}/\code{N} consecutive columns belong to the same subject
#' and the order of the column variables in each block is preserved across
#' subjects.
#'
#' For identifiability reasons, the trace of the row covariance matrix is set
#' equal to its dimension. If you want to place the equivalent restriction on
#' the column covariance matrix, interchange the role of row and column
#' variables by utilizing the function \code{\link{transposedata}}.
#'
#' @aliases covmat.hat print.covmat.hat
#' @param datamat numeric matrix containing the transposable data.
#' @param N positive integer number indicating the sample size, i.e., the
#' number of subjects.
#' @param shrink character indicating if shrinkage estimation should be
#' performed. Options include '\code{rows}', '\code{columns}', '\code{both}'
#' and '\code{none}'.
#' @param centered logical indicating if the transposable data are centered.
#' Options include \code{TRUE} or \code{FALSE}.
#' @param voi character indicating if the row, column or both covariance
#' matrices should be printed. Options include '\code{rows}', '\code{columns}'
#' and '\code{both}'.
#' @return Returns a list with components: \item{rows.covmat}{the estimated row
#' covariance matrix.} \item{rows.intensity}{the estimated row intensity.}
#' \item{cols.covmat}{the estimated column covariance matrix.}
#' \item{cols.intensity}{the estimated column intensity.} \item{N}{the sample
#' size.} \item{n.rows}{the number of row variables.} \item{n.cols}{the number
#' of column variables.} \item{shrink}{character indicating if shrinkage
#' estimation was performed.} \item{centered}{logical indicating if the
#' transposable data were centered.}
#' @references  Touloumis, A., Marioni, J. C. and Tavare, S. (2016)
#' HDTD: Analyzing  multi-tissue gene expression data,
#' \emph{Bioinformatics} \bold{32}, 2193--2195.
#' @author Anestis Touloumis
#' @examples
#' data(VEGFmouse)
#' # Estimating the gene and tissue covariance matrices.
#' est_cov_mat <- covmat.hat(datamat = VEGFmouse, N = 40)
#' est_cov_mat
#' @export covmat.hat
covmat.hat <- function(datamat, N, shrink = "both", centered = FALSE,
                        voi = "both") {
    if (!is.matrix(datamat))
        datamat <- as.matrix(datamat,
                            dimnames =
                                list(rownames(datamat), colnames(datamat)))
    datamat <- na.omit(datamat)
    rowsnam <- rownames(datamat)
    colsnam <- colnames(datamat)
    N <- as.numeric(N)
    if (length(N) != 1 | ((N - round(N)) != 0) | (N <= 0))
        stop("'N' must be a positive integer number")
    shrink <- as.character(shrink)
    shrinks <- c("none", "rows", "columns", "both")
    icheck <- as.integer(match(shrink, shrinks, -1))
    if (icheck < 1)
        stop("'shrink' must be one of 'rows', 'columns', 'none', 'both'")
    centered <- as.logical(centered)
    if (centered != TRUE & centered != FALSE)
        stop("'centered' must be either 'TRUE' or 'FALSE'")
    if (!centered & N <= 3)
        stop("'N' must be greater than or equal to 4")
    voi <- as.character(voi)
    vois <- c("rows", "columns", "both")
    icheck <- as.integer(match(voi, vois, -1))
    if (icheck < 1)
        stop(" 'voi' must be one of 'rows', 'columns', 'both'")
    p1 <- nrow(datamat)
    p2 <- ncol(datamat) / N
    if ((p2 - round(p2)) != 0)
        stop(" The number of column variables is not a positive integer number")
    colsnam <- colsnam[seq_len(p2)]
    pars <- covmat.hat.generic(datamat, N, shrink, centered, p1, p2, voi)
    shrink <- switch(shrink, none = "None", both = "Both sets of variables",
                        rows = "Row variables", columns = "Column variables")
    ans <- list(rows.covmat = pars$rowcovmat, rows.intensity = pars$lambdaS,
                cols.covmat = pars$colcovmat, cols.intensity = pars$lambdaD,
                N = N, n.rows = p1, n.cols = p2, shrink = shrink,
                centered = centered)
    if (!is.null(ans$rows.covmat))
        dimnames(ans$rows.covmat) <- list(rowsnam, rowsnam)
    if (!is.null(ans$cols.covmat))
        dimnames(ans$cols.covmat) <- list(colsnam, colsnam)
    class(ans) <- "covmat.hat"
    ans
}

#' Centering Transposable Data
#'
#' This function centers the transposable data around their sample mean matrix.
#'
#' It is assumed that there are \code{nrow(datamat)} row variables and
#' \code{ncol(datamat)}/\code{N} column variables in \code{datamat}. Further,
#' \code{datamat} should be written in such a way that every
#' \code{ncol(datamat)}/\code{N} consecutive columns belong to the same subject
#' and the order of the column variables in each block is preserved across
#' subjects.
#'
#' @param datamat numeric matrix containing the transposable data.
#' @param N positive integer number indicating the sample size, e.g., the
#' number of subjects.
#' @return Returns a matrix of the same size as \code{datamat}.
#' @author Anestis Touloumis
#' @seealso \code{\link{covmat.hat}} and \code{\link{covmat.ts}}.
#' @examples
#' data(VEGFmouse)
#' ## Centering the VEGF dataset around the sample mean matrix.
#' VEGFcen <- centerdata(datamat = VEGFmouse, N = 40)
#' @export centerdata
centerdata <- function(datamat, N) {
    if (!is.matrix(datamat))
        datamat <- as.matrix(datamat,
                            dimnames =
                                list(rownames(datamat), colnames(datamat)))
    datamat <- na.omit(datamat)
    N <- as.numeric(N)
    if (length(N) != 1 | ((N - round(N)) != 0) | (N <= 1))
        stop("'N' must be an integer number greater than 1")
    p1 <- nrow(datamat)
    p2 <- ncol(datamat) / N
    if ((p2 - round(p2)) != 0)
        stop("The number of column variables is not a positive integer number")
    meanmat <- sumdatamatrix(datamat, N) / N
    datamat - matrix(meanmat, p1, N * p2)
}

#' Estimation the Mean Matrix
#'
#' This function estimates the mean matrix.
#'
#' It is assumed that there are \code{nrow(datamat)} row variables and
#' \code{ncol(datamat)}/\code{N} column variables in \code{datamat}. Further,
#' \code{datamat} should be written in such a way that every
#' \code{ncol(datamat)}/\code{N} consecutive columns belong to the same subject
#' and the order of the column variables in each block is preserved across
#' subjects.
#'
#' @aliases meanmat.hat print.meanmat.hat
#' @param datamat numeric matrix containing the transposable data.
#' @param N positive integer number indicating the sample size, i.e., the
#' number of subjects.
#' @param group.sizes numeric vector indicating the size of the row or column
#' groups that share the same mean vector. It should be used only when
#' \code{group.vars=}'\code{rows}' or '\code{columns}'.
#' @param group.vars character indicating that the mean matrix can be
#' simplified over the row or column variables. Options include '\code{rows}'
#' or '\code{columns}'.
#' @return Returns a list with components: \item{estmeanmat}{the estimated mean
#' matrix.} \item{N}{the sample size.} \item{n.rows}{the number of row
#' variables.} \item{n.cols}{the number of column variables.}
#' @author Anestis Touloumis
#' @references  Touloumis, A., Marioni, J. C. and Tavare, S. (2016)
#' HDTD: Analyzing multi-tissue gene expression data.
#' \emph{Bioinformatics} \bold{32}, 2193--2195.
#' @examples
#' data(VEGFmouse)
#' ## The sample mean matrix of the VEGF mouse data.
#' sample_mean <- meanmat.hat(datamat = VEGFmouse, N = 40)
#' sample_mean
#' sample_mean$estmeanmat
#' @export
meanmat.hat <- function(datamat, N, group.sizes = NULL, group.vars = NULL) {
    if (!is.matrix(datamat))
        datamat <- as.matrix(datamat,
                             dimnames =
                                 list(rownames(datamat), colnames(datamat)))
    datamat <- na.omit(datamat)
    rowsnam <- rownames(datamat)
    colsnam <- colnames(datamat)
    N <- as.numeric(N)
    if (length(N) != 1 | ((N - round(N)) != 0) | (N <= 1))
        stop("'N' must be a positive integer number greater than 1")
    p1 <- nrow(datamat)
    p2 <- ncol(datamat) / N
    colsnam <- colsnam[seq_len(p2)]
    if ((p2 - round(p2)) != 0)
        stop("The number of column variables is not a positive integer number")
    ans <- sumdatamatrix(datamat, N)
    if (!is.null(group.sizes)) {
        group.sizes <- as.numeric(group.sizes)
        if (any(group.sizes <= 0 | (group.sizes - round(group.sizes)) != 0))
            stop("'group.sizes' must be a vector of positive integer numbers")
        if (group.vars != "columns" & group.vars != "rows")
            stop("'group.vars' must be either 'rows' or 'columns'")
        if (group.vars == "columns" & sum(group.sizes) != p2)
            stop("Total group sizes does not match number of column variables")
        if (group.vars == "rows" & sum(group.sizes) != p1)
            stop("Total group sizes does not match number of row variables")
        projmat <- projectionmatrix(cumsum(group.sizes))
        ans <- if (group.vars == "columns")
            ans %*% (diag(p2) - projmat) else (diag(p1) - projmat) %*% ans
    }
    ans <- ans / N
    dimnames(ans) <- list(rowsnam, colsnam)
    ans <- list(estmeanmat = ans, N = N, n.rows = p1, n.cols = p2)
    class(ans) <- "meanmat.hat"
    ans
}

covmat.ts.generic <- function(datamat, N, centered) {
    p1 <- nrow(datamat)
    p2 <- ncol(datamat) / N
    if (!centered) {
        test_statistics <- if (p1 <= p2)
            statistics(datamat, N) else statistics_trans(datamat, N)
        } else test_statistics <-
        if (p1 <= p2) {
            statistics_centered(datamat, N)
            } else {
            statistics_trans_centered(datamat, N)
            }
    Ustat <- p1 * test_statistics[2] / (test_statistics[1] ^ 2) - 1
    Vstat <- test_statistics[2] - 2 * test_statistics[1] + p1
    Dstat <- test_statistics[2] - test_statistics[3]
    trSigmaC2 <- test_statistics[4] / test_statistics[2]
    sigmaU <- (2 / (N - 1)) * (trSigmaC2 / (p2 ^ 2))
    sigmaV <- sigmaU * p1
    sigmaD <- sigmaU * test_statistics[3]
    Utest <- Ustat / sigmaU
    Vtest <- Vstat / sigmaV
    Dtest <- Dstat / sigmaD
    ans <- list(Utest = Utest, Vtest = Vtest, Dtest = Dtest)
    ans
}

meanmat.ts.generic <- function(datamat, N, projmat, voi) {
    if (voi == "columns")
        datamat <- transposedatamatrix(datamat, N)
    datamat <- projmat %*% datamat
    test_stats <- meanmatts_statistics(datamat, N)
    ans <- test_stats[1] / sqrt(2 * test_stats[2] / N / (N - 1))
    ans
}

covmat.hat.generic <- function(datamat, N, shrink, centered, p1, p2, voi) {
    lambdaS <- lambdaD <- 0
    datamat1 <- transposedatamatrix(datamat, N)
    if (!centered) {
        if (p1 < p2) {
            shrink_stats <- covmathat_statistics(datamat1, N)
            } else {
                shrink_stats <- covmathat_statistics_trans(datamat1, N)
                }
        } else {
            if (p1 < p2) {
                shrink_stats <- covmathat_statistics_centered(datamat1, N)
                } else {
                shrink_stats <- covmathat_statistics_trans_centered(datamat1, N)
                }
            }
    trDeltahat <- shrink_stats[1]
    trDelta2hat <- shrink_stats[2]
    trOmega2hat <- shrink_stats[3]
    trSigma2hat <- trOmega2hat / trDelta2hat
    if (centered) n <- N else n <- N - 1
    if (shrink == "both" | shrink == "columns") {
        lambdaD_a <- trSigma2hat / (p1 ^ 2) / n * (trDeltahat^2 + trDelta2hat)
        lambdaD_b <- trDelta2hat - trDeltahat ^ 2 / p2
        lambdaD <- lambdaD_a / (lambdaD_a + lambdaD_b)
        lambdaD <- max(0, min(1, lambdaD))
        }
    if (shrink == "both" | shrink == "rows") {
        lambdaS_a <- trDelta2hat / trDeltahat ^ 2 / n * (p1 ^ 2 + trSigma2hat)
        lambdaS_b <- trSigma2hat - p1
        lambdaS <- lambdaS_a / (lambdaS_a + lambdaS_b)
        lambdaS <- max(min(1, lambdaS), 0)
        }
    if (centered) {
        if (voi != "columns")
            Sigma <- tcrossprod(datamat) / N
        Delta <- tcrossprod(transposedatamatrix(datamat, N)) / N / p1
    } else {
        datacen <- centerdatamatrix(datamat, N)
        if (voi != "columns")
            Sigma <- matrix(0, p1, p1)
        Delta <- matrix(0, p2, p2)
        id <- rep(seq_len(N), each = p2)
        for (i in seq_len(N)) {
            if (voi != "columns")
                Sigma <- Sigma + tcrossprod(datacen[, id == i])
            Delta <- Delta + crossprod(datacen[, id == i])
        }
        if (voi != "columns")
            Sigma <- Sigma / (N - 1)
        Delta <- Delta / (N - 1) / p1
    }
    rowcovmat <- colcovmat <- NULL
    if (voi != "columns") {
        Sigma <- Sigma / trDeltahat
        rowcovmat <- Sigma * (1 - lambdaS) + diag(lambdaS, p1)
    }
    if (voi != "rows") {
        colcovmat <- Delta * (1 - lambdaD) + diag(lambdaD * trDeltahat / p2, p2)
    }
    if ((shrink == "none" | shrink == "rows") | voi == "rows")
        lambdaD <- NULL
    if ((shrink == "none" | shrink == "columns") | voi == "columns")
        lambdaS <- NULL
    ans <- list(rowcovmat = rowcovmat, lambdaS = lambdaS,
                colcovmat = colcovmat, lambdaD = lambdaD)
    ans
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

pmat <- function(x) {
    .Call(`_HDTD_pmat`, x)
}

projectionmatrix <- function(x) {
    .Call(`_HDTD_projectionmatrix`, x)
}

sumdatamatrix <- function(X, N) {
    .Call(`_HDTD_sumdatamatrix`, X, N)
}

centerdatamatrix <- function(X, N) {
    .Call(`_HDTD_centerdatamatrix`, X, N)
}

transposedatamatrix <- function(X, N) {
    .Call(`_HDTD_transposedatamatrix`, X, N)
}

tcrossprodcpp <- function(X) {
    .Call(`_HDTD_tcrossprodcpp`, X)
}

tcrossprod2cpp <- function(X, Y) {
    .Call(`_HDTD_tcrossprod2cpp`, X, Y)
}

crossprodcpp <- function(X) {
    .Call(`_HDTD_crossprodcpp`, X)
}

crossprod2cpp <- function(X, Y) {
    .Call(`_HDTD_crossprod2cpp`, X, Y)
}

sampleSigmaR <- function(X, N) {
    .Call(`_HDTD_sampleSigmaR`, X, N)
}

statistics_centered <- function(X, N) {
    .Call(`_HDTD_statistics_centered`, X, N)
}

statistics_trans_centered <- function(X, N) {
    .Call(`_HDTD_statistics_trans_centered`, X, N)
}

statistics <- function(X, N) {
    .Call(`_HDTD_statistics`, X, N)
}

statistics_trans <- function(X, N) {
    .Call(`_HDTD_statistics_trans`, X, N)
}

covmathat_statistics_centered <- function(X, N) {
    .Call(`_HDTD_covmathat_statistics_centered`, X, N)
}

covmathat_statistics_trans_centered <- function(X, N) {
    .Call(`_HDTD_covmathat_statistics_trans_centered`, X, N)
}

covmathat_statistics <- function(X, N) {
    .Call(`_HDTD_covmathat_statistics`, X, N)
}

covmathat_statistics_trans <- function(X, N) {
    .Call(`_HDTD_covmathat_statistics_trans`, X, N)
}

meanmatts_statistics <- function(X, N) {
    .Call(`_HDTD_meanmatts_statistics`, X, N)
}

projmatrix <- function(x) {
    lenx <- length(x)
    if (lenx != 1) {
        ans <- diag(x[lenx])
        index <- c(0, x)
        for (i in (seq_len(lenx) + 1)) {
            index1 <- seq(index[i - 1] + 1, index[i])
            lenind1 <- length(index1)
            if (lenind1 != 1) {
                ans[index1, index1] <-
                    ans[index1, index1] - rowMeans(ans[index1, index1])
            } else ans[index1, index1] <- 0
        }
    } else {
        ans <- diag(x) - 1 / x
    }
    ans
}

#' @export
print.meanmat.ts <- function(x, ...) {
    cat("MEAN MATRIX TEST", "\n")
    cat("Sample size      =", x$N, "\n")
    cat("Row variables    =", x$n.rows, "\n")
    cat("Column variables =", x$n.cols, "\n")
    if (x$n.groups > 1) {
        cat("\nH_0:", x$n.groups, "groups of", x$voi,
            "with a constant mean vector within each group \n")
    } else {
        cat("\nH_0: a constant mean vector across", x$voi,
            "\n")
    }
    cat("H_1: not H_0 \n")
    if (x$n.groups == 2)
        cat("\nThe number of ", x$voi,
            " in the two successive groups are ",
            paste(x$group.sizes, collapse = " and "),
            " respectively.\n", sep = "")
    if (x$n.groups > 2)
        cat("\nThe number of ", x$voi,
            " in the ", x$n.groups, " successive groups are ",
            paste(c(paste(x$group.sizes[seq_len(x$n.groups - 1)],
                    collapse = ", "), x$group.sizes[x$n.groups]),
                    collapse = " and "), " respectively.\n",
            sep = "")
    if (x$p.value < 1e-04)
        cat("\nTest statistic = ", round(x$statistic, 4),
            ", p-value < 0.0001\n", sep = "")
    if (x$p.value >= 1e-04)
        cat("\nTest statistic = ", round(x$statistic, 4),
            ", p-value = ", round(x$p.value, 4), "\n",
            sep = "")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/centerdata.R
\name{centerdata}
\alias{centerdata}
\title{Centering Transposable Data}
\usage{
centerdata(datamat, N)
}
\arguments{
\item{datamat}{numeric matrix containing the transposable data.}

\item{N}{positive integer number indicating the sample size, e.g., the
number of subjects.}
}
\value{
Returns a matrix of the same size as \code{datamat}.
}
\description{
This function centers the transposable data around their sample mean matrix.
}
\details{
It is assumed that there are \code{nrow(datamat)} row variables and
\code{ncol(datamat)}/\code{N} column variables in \code{datamat}. Further,
\code{datamat} should be written in such a way that every
\code{ncol(datamat)}/\code{N} consecutive columns belong to the same subject
and the order of the column variables in each block is preserved across
subjects.
}
\examples{
data(VEGFmouse)
## Centering the VEGF dataset around the sample mean matrix.
VEGFcen <- centerdata(datamat = VEGFmouse, N = 40)
}
\seealso{
\code{\link{covmat.hat}} and \code{\link{covmat.ts}}.
}
\author{
Anestis Touloumis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HDTD_data.R
\docType{data}
\name{VEGFmouse}
\alias{VEGFmouse}
\title{Vascular Endothelial Growth Factor Mouse Dataset}
\format{
A data frame with 46 rows and 360 columns. The rows corresponds to
46 genes in the VEGF signalling pathway. The column names indicate the mouse
and the tissue on which gene expression levels were measured.  Since there
are 40 mice and 9 tissues, we have a total of 360 columns. Every 9
consecutive columns belong to the same mouse and the tissues are ordered in
the same way in each mouse.
}
\source{
Zahn et al. (2007). AGEMAP: A gene expression database for aging in
mice. \emph{PLoS Genetics} \bold{3}, e201.
}
\description{
Log2 normalized mouse gene expression data in the vascular endothelial
growth factor signalling pathway across multiple tissues.
}
\examples{
data(VEGFmouse)
## Check the order of the tissues from the first mouse.
colnames(VEGFmouse[,1:9])
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meanmat.hat.R
\name{meanmat.hat}
\alias{meanmat.hat}
\alias{print.meanmat.hat}
\title{Estimation the Mean Matrix}
\usage{
meanmat.hat(datamat, N, group.sizes = NULL, group.vars = NULL)
}
\arguments{
\item{datamat}{numeric matrix containing the transposable data.}

\item{N}{positive integer number indicating the sample size, i.e., the
number of subjects.}

\item{group.sizes}{numeric vector indicating the size of the row or column
groups that share the same mean vector. It should be used only when
\code{group.vars=}'\code{rows}' or '\code{columns}'.}

\item{group.vars}{character indicating that the mean matrix can be
simplified over the row or column variables. Options include '\code{rows}'
or '\code{columns}'.}
}
\value{
Returns a list with components: \item{estmeanmat}{the estimated mean
matrix.} \item{N}{the sample size.} \item{n.rows}{the number of row
variables.} \item{n.cols}{the number of column variables.}
}
\description{
This function estimates the mean matrix.
}
\details{
It is assumed that there are \code{nrow(datamat)} row variables and
\code{ncol(datamat)}/\code{N} column variables in \code{datamat}. Further,
\code{datamat} should be written in such a way that every
\code{ncol(datamat)}/\code{N} consecutive columns belong to the same subject
and the order of the column variables in each block is preserved across
subjects.
}
\examples{
data(VEGFmouse)
## The sample mean matrix of the VEGF mouse data.
sample_mean <- meanmat.hat(datamat = VEGFmouse, N = 40)
sample_mean
sample_mean$estmeanmat
}
\references{
Touloumis, A., Marioni, J. C. and Tavare, S. (2016)
HDTD: Analyzing multi-tissue gene expression data.
\emph{Bioinformatics} \bold{32}, 2193--2195.
}
\author{
Anestis Touloumis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/covmat.ts.R
\name{covmat.ts}
\alias{covmat.ts}
\alias{print.covmat.ts}
\title{Nonparametric Tests for the Row or Column Covariance Matrix}
\usage{
covmat.ts(datamat = datamat, N = N, voi = "rows", centered = FALSE)
}
\arguments{
\item{datamat}{numeric matrix containing the transposable data.}

\item{N}{positive integer number indicating the sample size, i.e., the
number of subjects.}

\item{voi}{character indicating if the test should be applied on the row or
column covariance matrix. Options include '\code{rows}' or '\code{columns}'.}

\item{centered}{logical indicating if the transposable data are centered.
Options include \code{TRUE} or \code{FALSE}.}
}
\value{
It returns a list with components:
\item{diagonality.ts}{a list
containing the test statistic and p-value of the
diagonality hypothesis test.}
\item{sphericity.ts}{a list
containing the test statistic and p-value of the
sphericity hypothesis test.}
\item{identity.ts}{a list containing the test statistic and p-value of the
identity hypothesis test.}
\item{N}{the sample size.}
\item{n.rows}{the number of row
variables.}
\item{n.cols}{the number of column variables.}
\item{variables}{character indicating if the tests were applied to the row
or column covariance matrix.}
\item{centered}{logical indicating if the
transposable data were centered.}
}
\description{
Testing the sphericity, identity and diagonality hypotheses for
the row or column covariance matrix.
}
\details{
It is assumed that there are \code{nrow(datamat)} row variables and
\code{ncol(datamat)}/\code{N} column variables in \code{datamat}. Further,
\code{datamat} should be written in such a way that every
\code{ncol(datamat)}/\code{N} consecutive columns belong to the same subject
and the order of the column variables in each block is preserved across
subjects.

The tests are nonparametric and thus robust to some departures from the
matrix-variate normal model.
}
\examples{
data(VEGFmouse)
## Hypothesis tests for the covariance matrix of the genes (rows).
genes_cov_test <- covmat.ts(datamat = VEGFmouse, N = 40)
genes_cov_test
## Hypothesis tests for the covariance matrix of the tissues (columns).
tissues_cov_test <- covmat.ts(datamat = VEGFmouse, N = 40, voi = 'columns')
tissues_cov_test
}
\references{
Touloumis, A., Marioni, J.C. and Tavare, S. (2021) Hypothesis
Testing for the Covariance Matrix in High-Dimensional Transposable Data with
Kronecker Product Dependence Structure. \emph{Statistica Sinica} \bold{31},
1309--1329.
}
\author{
Anestis Touloumis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orderdata.R
\name{orderdata}
\alias{orderdata}
\title{Reordering Row and Column Variables}
\usage{
orderdata(datamat, N, order.rows = NULL, order.cols = NULL)
}
\arguments{
\item{datamat}{numeric matrix containing the transposable data.}

\item{N}{positive integer number indicating the sample size, i.e., the
number of subjects.}

\item{order.rows}{numeric vector displaying the desired order of the row
variables.}

\item{order.cols}{numeric vector displaying the desired order of the column
variables.}
}
\value{
Returns a matrix of the same size as \code{datamat}.
}
\description{
This utility function rearranges the row and/or the column variables in a
desired order.
}
\details{
It is assumed that there are \code{nrow(datamat)} row variables and
\code{ncol(datamat)}/\code{N} column variables in \code{datamat}. Further,
\code{datamat} should be written in such a way that every
\code{ncol(datamat)}/\code{N} consecutive columns belong to the same subject
and the order of the column variables in each block is preserved across
subjects.
}
\examples{
data(VEGFmouse)
set.seed(1)
tissuesold <-  colnames(VEGFmouse[ ,1:9])
## Suppose that you want to order the tissues in the folowing order.
tissuesnew <- colnames(VEGFmouse[ ,1:9])[sample(9)]
tissuesnew
## To do this, create a numeric vector with the desired order.
ordtis <- pmatch(tissuesnew, tissuesold)
VEGFmousenew <- orderdata(datamat = VEGFmouse, N = 40, order.cols = ordtis)
colnames(VEGFmousenew)[1:9]
}
\seealso{
\code{\link{meanmat.ts}} and \code{\link{meanmat.hat}}.
}
\author{
Anestis Touloumis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transposedata.R
\name{transposedata}
\alias{transposedata}
\title{Interchanging the Row and Column Variables in Transposable Data}
\usage{
transposedata(datamat, N)
}
\arguments{
\item{datamat}{numeric matrix containing the transposable data.}

\item{N}{positive integer number indicating the sample size, i.e., the
number of subjects.}
}
\value{
Returns a matrix with \code{ncol(datamat)} rows and
\code{nrow(datamat)}/\code{N} columns.
}
\description{
This function interchanges the row and column variables in transposable data
so that the original row variables will be treated as column variables and
the original column variables as row variables.
}
\details{
It is assumed that there are \code{nrow(datamat)} row variables and
\code{ncol(datamat)}/\code{N} column variables in \code{datamat}. Further,
\code{datamat} should be written in such a way that every
\code{ncol(datamat)}/\code{N} consecutive columns belong to the same subject
and the order of the column variables in each block is preserved across
subjects.
}
\examples{
data(VEGFmouse)
## Transposing the VEGF dataset.
VEGFtr <- transposedata(datamat = VEGFmouse, N = 40)
}
\seealso{
\code{\link{centerdata}} and \code{\link{orderdata}}.
}
\author{
Anestis Touloumis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meanmat.ts.R
\name{meanmat.ts}
\alias{meanmat.ts}
\alias{print.meanmat.ts}
\title{Nonparametric Tests for the Mean Matrix}
\usage{
meanmat.ts(datamat, N, group.sizes, voi = "columns")
}
\arguments{
\item{datamat}{numeric matrix containing the transposable data.}

\item{N}{positive integer number indicating the sample size, i.e., the
number of subjects.}

\item{group.sizes}{numeric vector indicating the group sizes under the null
hypothesis.}

\item{voi}{character indicating if the test will be applied to the row or
column variables. Options include '\code{rows}' or '\code{columns}'.}
}
\value{
Returns a list with components: \item{statistic}{the value of the
test statistic.} \item{p.value}{the corresponding p-value.} \item{voi}{the
set of variables that the test was applied to.} \item{n.groups}{the number
of groups under the null hypothesis.} \item{group.sizes}{the size of each
group under the null hypothesis.} \item{N}{the sample size.}
\item{n.rows}{the number of row variables.} \item{n.cols}{the number of
column variables.}
}
\description{
This function performs hypothesis testing for the mean matrix.
}
\details{
It is assumed that there are \code{nrow(datamat)} row variables and
\code{ncol(datamat)}/\code{N} column variables in \code{datamat}. Further,
\code{datamat} should be written in such a way that every
\code{ncol(datamat)}/\code{N} consecutive columns belong to the same subject
and the order of the column variables in each block is preserved across
subjects.
}
\examples{
data(VEGFmouse)
## Testing conservation of the overall gene expression across tissues.
tissues_mean_test <- meanmat.ts(datamat = VEGFmouse, N = 40, group.sizes = 9)
tissues_mean_test
## Testing if the adrenal and the cerebrum tissues have the same mean vector.
test2 <- meanmat.ts(VEGFmouse, N = 40, group.sizes = c(2, rep(1,7)))
test2
}
\references{
Touloumis, A., Tavare, S. and Marioni, J. C. (2015) Testing the
Mean Matrix in High-Dimensional Transposable Data.
\emph{Biometrics} \bold{71}, 157--166.
}
\author{
Anestis Touloumis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HDTD-package.R
\docType{package}
\name{HDTD-package}
\alias{HDTD-package}
\alias{HDTD}
\title{Estimation and Hypothesis Testing in High-Dimensional Transposable Data}
\description{
The package HDTD offers functions to estimate and test the matrix parameters
of transposable data in high-dimensional settings.
}
\details{
The term transposable data refers to datasets that are structured in a
matrix form such that both the rows and columns correspond to variables of
interest. For example, consider microarray studies in genetics where
multiple RNA samples across different tissues are available per subject. In
this case, a data matrix can be created with row variables the genes, column
variables the tissues and measurements the corresponding expression levels.

The function \code{\link{meanmat.hat}} estimates the mean matrix of the
transposable data.

The mean relationship of the row and column variables can be tested using
the function \code{\link{meanmat.ts}}. The implemented test is nonparametric
and not seriously restricted by the dependence structure among and/or
between the row and column variables.
See \cite{Touloumis et al. (2015)} for more details.

The function \code{\link{covmat.hat}} provides Stein-type shrinkage
estimators for the row covariance matrix and/or for the column covariance
matrix under a matrix-variate normal model.
See \cite{Touloumis et al. (2016)} for more details.

The sphericity and identity hypothesis for the row or column covariance
matrix can be tested using the function \code{\link{covmat.ts}}. Both tests
are nonparametric, i.e., they do not rely on a normality assumption.
See \cite{Touloumis et al. (2017)} for more details.

There are three utility functions that allow the user to change to
interchange the role of row and column variables
(\code{\link{transposedata}}), to center the transposable data
(\code{\link{centerdata}}) or to rearrange the order of the row and/or
column variables (\code{\link{orderdata}}).
}
\examples{
data(VEGFmouse)
## The sample mean matrix.
sample_mean <- meanmat.hat(datamat = VEGFmouse, N = 40)
sample_mean
## Testing conservation of the overall gene expression across tissues.
tissues_mean_test <- meanmat.ts(datamat = VEGFmouse, N = 40, group.sizes = 9)
tissues_mean_test
# Estimating the gene and column covariance matrices.
est_cov_mat <- covmat.hat(datamat = VEGFmouse, N = 40)
est_cov_mat
## Hypothesis tests for the covariance matrix of the genes (rows).
genes_cov_test <- covmat.ts(datamat = VEGFmouse, N = 40)
genes_cov_test
## Hypothesis tests for the covariance matrix of the tissues (columns).
tissues_cov_test <- covmat.ts(datamat = VEGFmouse, N = 40, voi = 'columns')
tissues_cov_test
}
\references{
Touloumis, A., Tavare, S. and Marioni, J. C. (2015) Testing the
Mean Matrix in High-Dimensional Transposable Data.
\emph{Biometrics} \bold{71}, 157--166.

Touloumis, A., Marioni, J. C. and Tavare, S. (2016) HDTD: Analyzing
multi-tissue gene expression data. \emph{Bioinformatics}
\bold{32}, 2193--2195.

Touloumis, A., Marioni, J. C. and Tavare, S. (2021) Hypothesis Testing for
the Covariance Matrix in High-Dimensional Transposable Data with Kronecker
Product Dependence Structure. \emph{Statistica Sinica} \bold{31}, 1309--1329.
}
\author{
Anestis Touloumis, John Marioni, Simon Tavare.

Maintainer: Anestis.Touloumis <A.Touloumis@brighton.ac.uk>
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/covmat.hat.R
\name{covmat.hat}
\alias{covmat.hat}
\alias{print.covmat.hat}
\title{Estimation of the Row and of the Column Covariance Matrices.}
\usage{
covmat.hat(datamat, N, shrink = "both", centered = FALSE, voi = "both")
}
\arguments{
\item{datamat}{numeric matrix containing the transposable data.}

\item{N}{positive integer number indicating the sample size, i.e., the
number of subjects.}

\item{shrink}{character indicating if shrinkage estimation should be
performed. Options include '\code{rows}', '\code{columns}', '\code{both}'
and '\code{none}'.}

\item{centered}{logical indicating if the transposable data are centered.
Options include \code{TRUE} or \code{FALSE}.}

\item{voi}{character indicating if the row, column or both covariance
matrices should be printed. Options include '\code{rows}', '\code{columns}'
and '\code{both}'.}
}
\value{
Returns a list with components: \item{rows.covmat}{the estimated row
covariance matrix.} \item{rows.intensity}{the estimated row intensity.}
\item{cols.covmat}{the estimated column covariance matrix.}
\item{cols.intensity}{the estimated column intensity.} \item{N}{the sample
size.} \item{n.rows}{the number of row variables.} \item{n.cols}{the number
of column variables.} \item{shrink}{character indicating if shrinkage
estimation was performed.} \item{centered}{logical indicating if the
transposable data were centered.}
}
\description{
This function provides the row and/or column covariance matrix estimators.
}
\details{
It is assumed that there are \code{nrow(datamat)} row variables and
\code{ncol(datamat)}/\code{N} column variables in \code{datamat}. Further,
\code{datamat} should be written in such a way that every
\code{ncol(datamat)}/\code{N} consecutive columns belong to the same subject
and the order of the column variables in each block is preserved across
subjects.

For identifiability reasons, the trace of the row covariance matrix is set
equal to its dimension. If you want to place the equivalent restriction on
the column covariance matrix, interchange the role of row and column
variables by utilizing the function \code{\link{transposedata}}.
}
\examples{
data(VEGFmouse)
# Estimating the gene and tissue covariance matrices.
est_cov_mat <- covmat.hat(datamat = VEGFmouse, N = 40)
est_cov_mat
}
\references{
Touloumis, A., Marioni, J. C. and Tavare, S. (2016)
HDTD: Analyzing  multi-tissue gene expression data,
\emph{Bioinformatics} \bold{32}, 2193--2195.
}
\author{
Anestis Touloumis
}
